
# q-expansion of newform 275.4.a.g, downloaded from the LMFDB on 03 January 2025.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField, ZZ
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K([c/ZZ(den) for c in num]) for num, den in basis_data]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 275
weight = 4
poly_data = [-148, 304, 61, -38, -2, 1]

# The entries in the following list give a basis for the
# coefficient ring in terms of a root of the defining polynomial above.
# Each line consists of the coefficients of the numerator, and a denominator.
basis_data  = [[[1, 0, 0, 0, 0], 1], [[0, 1, 0, 0, 0], 1], [[-18, -87, -22, 4, 1], 4], [[-46, 87, 26, -4, -1], 4], [[-82, 165, 74, -8, -3], 4]]

hecke_ring_character_values = None
aps_data = [[0, -1, 0, 0, 0], [0, 1, 0, 0, 1], [0, 0, 0, 0, 0], [-9, 2, -2, -1, 0], [-11, 0, 0, 0, 0], [-41, -2, 2, -4, -2], [-22, 3, -6, -8, -17], [-1, -4, -6, 2, -14], [-7, -15, 4, 2, -5], [-32, 24, 18, 15, 34], [-5, 28, 16, -20, -2], [-177, 9, 4, -3, -9], [13, -7, -30, -21, -17], [-78, -21, 16, 33, 41], [-17, 17, 24, -13, -31], [-61, 64, 22, 31, 48], [249, 23, 56, 25, 43], [179, -70, 12, 17, 60], [4, -94, -28, 46, 2], [-126, -71, -16, 1, -41], [-285, -60, -60, -5, -14], [28, -67, 0, -56, -77], [76, 68, -52, -21, -100], [542, -78, -86, -15, -98], [-149, 125, 58, -16, 23], [-845, 93, -8, -4, 33], [80, -200, -60, -45, 54], [-355, 216, -10, 70, 198], [-62, 186, -66, -13, 136], [407, 79, 96, -57, 61], [-1128, -97, 56, -32, -15], [-343, 315, 62, 32, 75], [-388, -46, -154, 23, -22], [1631, -42, 46, -60, -52], [-448, -250, 16, -134, 70], [-825, -299, -110, 23, -119], [-866, 304, 32, -4, -216], [-127, -254, 44, 167, -306], [-129, -241, -2, -101, 219], [-525, 152, -70, 220, 168], [271, 224, -164, 173, 104], [709, 374, -2, -105, 34], [-620, 244, -368, -131, -286], [-2180, 772, 240, 260, 408], [-677, -637, 84, -28, -201], [170, -6, -24, -257, 64], [-1229, 333, 138, 7, 33], [367, 17, -84, 83, 177], [-592, 204, 64, -253, 228], [-52, -135, 214, -106, 135], [-11, 234, 92, 471, 320], [-2407, 326, 62, -105, -704], [-394, -197, -310, -6, 303], [-832, -219, -468, -470, -819], [1418, -333, 144, 217, -271], [-1849, 113, -122, 179, 357], [-1250, 741, -110, -32, -249], [-2203, -9, 70, 83, -429], [-787, -205, 214, 557, 289], [3615, -25, -374, -159, -275], [40, -31, 620, 320, 655], [-30, 516, -400, -340, 296], [-3130, 323, -240, 110, -267], [-1567, 414, 172, -156, -596], [-2602, -1082, 224, 392, -586], [-3372, -263, 234, -428, 43], [-3535, -1071, 484, 501, 169], [3090, 104, 520, 100, -420], [3651, -526, 398, 259, 548], [-1417, -1922, -694, 370, -702], [4919, -428, -288, -24, 1116], [3990, -2190, -436, -326, -810], [1391, 67, 128, -726, -951], [-3597, 667, 194, 167, 553], [-973, 693, 488, 357, 1533], [8338, -619, -376, 287, -473], [-2756, -1148, 688, 384, 792], [158, -1321, -326, -88, -247], [2510, -1561, -20, -303, 569], [-371, 1459, -670, -713, 129], [2897, -689, -284, -381, 911], [-3671, 822, -278, 113, -178], [-939, 1081, -426, -123, 1681], [-6599, -858, -712, -526, -1078], [-2334, 511, 108, -254, -139], [5084, 478, -428, -1004, -558], [-2607, -1049, 294, 732, 421], [1341, 1142, -112, 149, -916], [3929, -763, -198, -727, 207], [9491, 230, -272, -66, 904], [8463, 315, 264, 637, 1379], [-2097, -1981, -78, 1069, -1061], [2709, 114, 1672, 746, 1784], [245, 378, 626, 420, 1148], [1411, 2282, -92, -1, 1622], [-6670, 146, -280, -480, 682], [-4471, -1752, -594, -255, 128], [188, -1789, 416, 279, 561], [-2002, 1825, 1424, 1347, 1639], [3609, 843, -444, 120, -657], [-1511, -1723, -178, -34, -439], [-2713, 1124, -1374, 1198, 1640], [7807, -744, 766, 1063, -834], [-4267, -448, 1360, -559, 122], [3953, -1343, 1094, 120, 1257], [-5974, -2251, -322, 424, 95], [1422, -785, -1724, -1892, -733], [-4349, -3103, 498, 939, -2237], [3042, -1747, 1940, 204, 1833], [1761, 1482, 420, 1181, 2172], [-10830, 200, -100, -740, 1128], [-10138, 1757, -914, -802, -1971], [772, 2925, 1456, -1667, 63], [-819, -929, -588, -1355, -329], [8551, 528, 360, 269, 2208], [-459, -512, 1468, -1528, -2192], [-5718, -1210, -1244, -482, -2178], [-127, 1437, 484, 457, -659], [1640, 1017, 1610, 350, -333], [2100, 272, 336, -1283, 892], [7253, -3029, -1072, -109, -2299], [-10671, 3257, -418, 711, 995], [-5020, -2373, 440, 335, -331], [-1499, 1035, -692, -3051, -1361], [-1533, 2538, 772, 981, 658], [11784, 517, 1064, 635, -593], [-2356, -40, -1988, -508, 1580], [17227, -1609, -240, 563, 71], [11078, -3140, -1396, -223, -3894], [-9811, 1050, 1962, 1086, 3570], [-4433, 3602, -334, -73, 2792], [3159, 2008, -2158, -2139, -830], [-2616, 3468, 556, 1745, 1058], [-5141, -2233, -888, -359, -3775], [-7944, 4536, -1188, -840, -3204], [-1005, -923, 162, 783, -373], [-5651, 4144, -1698, 331, 1752], [1873, 1445, 34, -223, -1479], [1298, 3967, 1934, 362, 3029], [-11532, 810, -1492, -1542, -2630], [-7445, -523, -366, -2305, -1463], [-18348, -2725, 740, 1805, 25], [-512, -3696, 2344, -468, 764], [-5504, -5969, -732, 519, -879], [-11116, 185, 1666, -1654, -345], [-8239, -874, -184, -1881, -3114], [438, 673, 414, -298, 245], [-12585, -4579, -3470, 765, -1277], [1766, 6694, -1712, 760, 2534], [22677, -1048, 1076, 978, -66], [-17636, 3183, 592, -1759, -4567], [14863, -3979, -226, -638, -2069], [7821, -3663, -3032, 389, -6119], [1701, -1987, 2238, 2599, 2357], [9772, 5042, -1004, -2202, 378], [4286, -3210, 544, 3034, 1490], [-5976, -620, -1176, -4056, -5140], [-3115, 308, 900, -626, -112], [-4832, -30, 1124, -1858, 910], [1757, 59, 2918, 2665, 2409], [-22027, 4961, 1364, -913, 205], [-11298, -5088, 1376, 1308, -3900], [-8720, 4321, -1240, 750, 1659], [-13897, 6562, 884, 2055, 3382], [2527, -4793, 496, 23, -3007], [21588, 101, 2304, 2987, 2499], [13602, -9351, -1832, -840, -931], [-13476, -1923, 2682, 5276, 3153], [6423, -2597, 854, 1363, 3113], [-21616, -4040, -2128, 1206, -2808], [6533, -1445, 2398, 148, 4235], [-25561, 210, -1406, -2231, 2050], [-2108, 1031, 680, 678, -579], [1489, -7280, 472, -1459, 2], [-7779, 2508, 496, 310, -502], [-8082, 1733, -1604, -565, -1477], [15868, 1675, -2884, -1759, -5415], [-9489, 29, 116, -3797, 2379], [-15674, 298, 1508, 184, -3798], [-484, -4281, -1588, -551, -4811], [-19252, 5081, -2396, -643, 1019], [-12297, -1844, -2946, 1627, -1754], [-23753, -263, 716, -67, 4371], [15374, -4308, -1408, 636, -3540], [-13087, -6857, 1404, 2557, 2263], [35693, -2067, 3480, 739, 2103], [4870, 3455, -1280, -5205, 1069], [-16981, -6496, 3292, 3341, 3960], [-24141, 1729, -4338, -841, -1841], [-2596, -156, 1152, -2616, -4436], [-7632, 3694, -3426, 2717, 2546], [16348, -11241, -4676, -963, -7295], [4795, -411, 3368, 4015, 529], [6270, 2405, -2110, 2490, -4625], [-21369, 5787, 3478, 1134, 4843], [27741, 7680, 4568, -146, 4776], [13304, 566, 2712, 2780, 2086], [-11283, 6341, 5156, -227, 8099], [19827, -3766, -2404, 223, -4772], [5192, 7015, 2676, -312, 569], [-13183, 4561, -3256, -1018, 1801], [25467, -2670, 2506, 691, 5540], [-28944, 2735, -2222, -906, -2415], [4608, -3105, -906, -2226, -1485], [33974, -2140, 2688, 2308, -3220], [-10027, 814, 614, 2927, -122], [-16660, -4136, -244, 3948, -1816], [-54737, 2984, 2134, -168, 2046], [13578, -1034, -2072, 966, -474], [-17065, 140, 452, -2323, 2260], [895, 6673, -1830, -2975, -4825], [-77, 3217, -3484, 1957, 2087], [10487, -3134, -4204, -227, 2458], [-16396, 3404, -488, -3274, 5424], [36961, 2011, -2202, -1809, 891], [16298, 4075, 1392, -1413, -1455], [9364, 262, 1872, 1446, 7418], [1522, 4115, -1186, -216, -2085], [-10817, 14749, 1344, 1792, 7619], [13655, 6924, 1830, -3815, 4276], [-11948, 7373, 4438, 572, -1747], [31737, -1225, 2790, 2845, 795], [-28604, -7571, 2784, -509, -3741], [-15930, 2765, 4220, -4980, 1673], [6361, -3279, -542, -1511, 5817], [-7893, 1260, -1338, -5858, -5160], [53755, -3162, 1050, 2935, 4954], [6596, -5089, 5804, 3107, 1381], [25696, -7328, -5492, -6896, -9872], [-48594, 5204, 132, 1359, 4904], [-14597, -7504, -1426, 1297, -13384], [-10523, -7862, 1878, -1178, 368], [27713, -9687, -4774, 95, -10927], [-10289, -1797, -1778, -4029, 733], [-19562, 4889, 2064, -3058, -6111], [-55991, -2649, -1298, -39, -3069], [-36626, -2931, 2396, 3313, -2261], [-675, 7795, -4700, -350, 3479], [-23732, -4065, -5236, 3924, 2495], [26783, 4358, -5864, -4815, -6002], [18781, -2855, 1708, -611, 5209], [31146, 1416, -5196, -2018, -9024], [-22914, 6185, 3048, 3929, -2273], [-7869, -6591, -5340, -5773, -961], [17602, 2928, 7016, 2158, 2844], [-663, 3810, -6098, 927, -1680], [-1479, -2263, 1148, -4036, 2247], [-13787, 5592, 244, -4645, -7708], [-9296, 4088, 2928, 8077, 6088], [-26864, 7168, -852, -3826, 1248], [-24676, -6519, -7468, -499, -7765], [-2275, -3947, -1086, -3437, -7077], [3302, -4995, -1464, 4553, -3121], [-15765, 1744, 3680, 9332, 11604], [-28297, -1145, -2036, -4338, -7225], [-8158, 12259, -2540, -5513, -4651], [13257, -11905, -3232, -2437, -5575], [-31107, 5171, 4334, 257, -3969], [-2151, 1986, 1502, -1729, 222], [-59626, -6848, -1408, -4284, -4064], [-7648, -8172, 1156, 4948, 5532], [37146, 12321, -2412, 1869, 3171], [50625, 9306, -3998, -5830, -2854], [-13560, -6150, 4810, 2375, -5336], [-2411, 14514, 672, -2284, -3466], [35754, 1812, 80, 7866, 3272], [-17443, 187, 4616, 8157, 7277], [3101, 4095, 1322, -9268, -2755], [-45345, 1736, 2570, -1170, -1466], [29891, -181, 1658, -4301, 345], [-54456, -4654, 104, -1788, -9874], [-10103, 12567, 4756, 5483, 11879], [34984, 1745, 1152, 2241, -293], [-7513, 6175, -4622, 3683, 10015], [4558, 2569, 2754, -5018, 2597], [-16313, -3028, 7156, 15588, 12806], [31436, 6724, -3812, 7924, 4712], [57136, -3811, 890, 4152, -211], [9667, 7878, 10924, 6103, 18488], [-931, -1085, 7462, 917, 2675], [-9402, -14743, -3176, -1228, -10901], [-18100, 24179, 208, -935, -2631], [-16319, -11972, 2918, -281, -5226], [-37425, -21332, 3646, 7390, 2488], [-39579, 8789, 2198, 4884, 14909], [22540, -4827, 1460, 3539, 7643], [52284, 8884, -5998, 4771, -5208], [-14837, -5803, -5756, -7458, -6471], [-21051, -4685, -7568, -3028, 2005], [-30374, -4267, 7966, -2200, 4673], [28613, -3731, 3864, 5631, -2121], [14405, -15275, 6640, 4775, -2535], [30344, 264, -3148, 4616, -1240], [7266, 15564, 6136, -5097, 13184], [29123, -9916, -5866, -558, -7216], [44325, 4615, -3000, -6450, -5645], [24567, -9764, 1740, -1252, 6986], [74269, 12235, -4828, -3289, 3329], [11770, 17713, 8780, -2841, -497], [-4719, 11106, -7742, -766, -10168], [-7317, -1949, 714, -483, -5571], [-1014, 22905, -9312, -681, -6405], [-23021, -3962, -6898, -9439, 5428], [-33195, -10465, -11200, -4475, -4777], [-17118, -7495, 1916, 7106, -13555], [28365, -7803, -4920, -415, -3553], [-1720, 10462, -6240, -6800, 9642], [29987, -3222, -4722, -6233, -17622], [32340, -2064, -8740, 2483, -6094], [-18639, -12194, 4798, -7416, 2650], [-10566, -1103, 5692, -6844, -1117], [67166, -2249, 1166, 6784, 5281], [-25065, -14772, 5812, 7771, -13022], [14875, -4271, -800, -4425, 1257], [8511, 22609, 7018, -4511, 11141], [11690, -21660, -7536, -726, -6600], [-17602, -9618, -2676, -1470, -23398], [-66738, 8782, -10196, -5060, -3738], [31192, -2408, -8244, -6582, -14188], [-1269, 15372, -2384, 4857, -6778], [-17033, -30106, 2586, -917, -7696], [-27331, -3821, -7118, -914, -7807], [-34871, 6827, 6230, 8601, 2357], [-35873, 5968, 2316, -1622, -5470], [133298, -4181, 7594, 4742, 9347], [19226, -1211, 2108, -4445, -14001], [13116, 931, 5272, 1180, 6331], [-112005, -4366, -2540, -10455, 474], [-3420, 17202, 7240, 4960, 5338], [-55742, 16763, 12644, 7637, 21625], [-24254, -10609, 5016, 739, -10019], [-13067, -10803, 2704, -4813, 8099], [-102860, 2557, -6688, -3272, -513], [-65027, 21426, 7314, 8912, 20112], [13030, -1253, 11140, 14151, 9417], [12966, -10988, 2268, 4204, -3224], [89424, -12998, -2548, -1172, 5322], [11730, 5642, -4480, -4040, -9314], [60839, -13134, 4158, -1049, -10154], [-44293, 6660, -15514, -4777, -19874], [-108622, 16908, 6844, 922, 4076], [77773, 3636, -576, -828, -10912], [24231, 13977, 8046, 6727, 16727], [14701, 21947, -4144, 1932, 9737], [33257, 7691, 56, -9107, -6051], [-33950, -16585, -7664, -5034, -11855], [-53126, 13167, 6078, 7964, 17637], [1617, 25471, 8096, 3043, 6711], [77074, 24561, 11060, -672, 8771], [-14261, 6991, 3132, 10301, 6009], [83557, 10897, -4422, 1984, 17757], [8579, -19518, -15258, -11689, -6246], [-23943, 11679, -4608, -2301, 15969], [-23768, -23321, -8944, 5258, -19359], [37928, -10475, 1520, 3115, -525], [-31825, 4601, 3870, 4925, -1889], [72714, 19258, 10372, -2414, 2138], [17279, 2953, 9594, 10798, 21693], [36283, -3163, 1594, -6733, 4879], [65740, -4081, 2000, 825, 2481], [-79235, 6949, -6240, 3460, 3389], [34981, 1239, 15138, 15625, 3209], [6157, -30726, -6544, -6712, -3552], [-39692, 4998, -2884, 1815, -1402], [-17541, -22454, 6012, 261, 2618], [-62683, 7946, -5204, -8132, -6030], [-79560, 10276, -7320, 1050, -2852], [-71145, -7199, -13850, -5015, -15063], [56628, -19264, -10232, -2504, 8776], [-57836, 10179, 9312, 9724, -12221], [119076, -1944, 16984, 1592, 56], [-40883, -20828, -13154, 1183, -5158], [25522, 18557, -750, 6026, 10037], [29958, 20393, -6668, -3584, 23743], [-57569, -9545, 11468, 15109, 17025], [-10349, 12766, 780, 7983, 22506], [55713, 6966, -866, 3697, 17376], [16494, -1154, 7892, -6394, -5366], [-139265, -740, -21720, -4505, -5690], [13041, 356, 9108, -1441, -4418], [-69620, 17320, 3108, 1858, 1780], [-82297, -19681, -936, -6573, 4533], [-42030, 8308, -9260, -2430, 2320], [80270, 27007, 10110, -5120, 1255], [-71233, -26043, 3418, 11119, -2203], [-3144, -2648, -11492, -4981, -27758], [15957, -19561, 640, 10242, 7109], [26240, 5142, 60, 11050, 546], [104883, 6896, 7294, 8692, 16830], [62970, -23490, -220, 2050, -17874], [5375, 42099, 4186, 9118, 1429], [51824, -3348, -1508, 9336, 10572], [-22485, 25265, 20, -2885, -1735], [57680, -8472, -9020, -12410, -16276], [1441, 36836, 11928, 3146, -94], [64695, 26773, -8750, -12185, 8383], [35072, 179, -8864, -11822, -2211], [20770, 3522, -2340, -16364, -9038], [37029, 12163, 476, 4445, -5017], [-51119, 25445, -4732, -8012, -9735], [42780, 23549, 10848, -2005, 3739], [132691, -10201, 13898, 2624, 8913], [107221, -13788, 1698, 6939, 24102], [-51865, 16070, -380, -15730, -8226], [66705, -10098, 2930, -8394, -13718], [-2659, -7067, -11326, -7657, -15987], [21622, 40716, 13176, -4562, 2780], [-21819, -10463, -4210, -5499, 3827], [31436, 12034, -10172, 10414, -5438], [45664, 14356, -8540, 9478, 9396], [-2530, 29013, 1270, -6410, 11621], [-59445, -2800, 10310, -2290, -16988], [19313, -5381, 7404, 10127, 16495], [-15025, -27855, -11518, -6038, -11935], [-23910, -11202, -18860, -15230, -33306], [-106771, 6647, -4268, -10377, -33943], [-5165, 15107, 11162, 17933, -393], [8940, 13065, -4260, -430, -3363], [68939, -10116, -13588, -2654, 11812], [-66456, -16953, 952, -5154, 3573], [-40543, -26173, 8612, 13143, 11937], [-170795, -12046, -6210, -7050, -2402], [55083, 8184, 11524, 2482, 8886], [55983, -30312, -12546, -13552, 1078], [-18074, -3229, 20108, 16679, 36473], [-7916, -14200, -9808, -264, -5212], [87186, 9090, 9388, -5206, -590], [33898, -30347, -11942, -168, -17347], [-61698, 5714, 3416, -6662, -9286], [37672, -5493, -9624, -10018, 4037]]
